package orderservicelogic

import (
	"app-cloudep-order-server/gen_result/pb/order"
	mocksvc "app-cloudep-order-server/internal/mock/lib"
	mockmodel "app-cloudep-order-server/internal/mock/model"
	"app-cloudep-order-server/internal/svc"
	"context"
	"errors"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.uber.org/mock/gomock"
)

func TestDeleteOrder(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockOrderModel := mockmodel.NewMockOrderModel(ctrl)
	mockValidate := mocksvc.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		OrderModel: mockOrderModel,
		Validate:   mockValidate,
	}

	// 測試數據集
	tests := []struct {
		name      string
		input     *order.DeleteOrderReq
		prepare   func()
		expectErr bool
	}{
		{
			name: "成功刪除訂單",
			input: &order.DeleteOrderReq{
				BusinessId: "B123",
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬資料庫刪除成功
				mockOrderModel.EXPECT().DeleteByBusinessID(gomock.Any(), "B123").Return(nil, nil).Times(1)
			},
			expectErr: false,
		},
		{
			name: "驗證失敗",
			input: &order.DeleteOrderReq{
				BusinessId: "B123",
			},
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("validation failed")).Times(1)
			},
			expectErr: true,
		},
		{
			name: "刪除資料庫失敗",
			input: &order.DeleteOrderReq{
				BusinessId: "B123",
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬資料庫刪除失敗
				mockOrderModel.EXPECT().DeleteByBusinessID(gomock.Any(), "B123").Return(nil, errors.New("delete failed")).Times(1)
			},
			expectErr: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 設置測試環境
			tt.prepare()

			// 初始化 DeleteOrderLogic
			logic := NewDeleteOrderLogic(context.TODO(), svcCtx)

			// 執行 DeleteOrder
			resp, err := logic.DeleteOrder(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, resp)
			} else {
				assert.NoError(t, err)
				assert.NotNil(t, resp)
			}
		})
	}
}
