package orderservicelogic

import (
	"app-cloudep-order-server/gen_result/pb/order"
	mocksvc "app-cloudep-order-server/internal/mock/lib"
	mockmodel "app-cloudep-order-server/internal/mock/model"
	"app-cloudep-order-server/internal/svc"
	"context"
	"errors"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.uber.org/mock/gomock"
)

func TestModifyOrderStatus(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockOrderModel := mockmodel.NewMockOrderModel(ctrl)
	mockValidate := mocksvc.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		OrderModel: mockOrderModel,
		Validate:   mockValidate,
	}

	// 測試數據集
	tests := []struct {
		name        string
		input       *order.ModifyOrderStatusReq
		prepare     func()
		expectErr   bool
		expectedRes *order.OKResp
	}{
		{
			name: "成功修改訂單狀態",
			input: &order.ModifyOrderStatusReq{
				BusinessId: "B123",
				Status:     3,
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬狀態更新成功
				mockOrderModel.EXPECT().UpdateStatus(gomock.Any(), gomock.Any()).Return(nil, nil).Times(1)
			},
			expectErr:   false,
			expectedRes: &order.OKResp{},
		},
		{
			name: "驗證失敗",
			input: &order.ModifyOrderStatusReq{
				BusinessId: "B123",
				Status:     12, // 錯誤的狀態值
			},
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("invalid status")).Times(1)
			},
			expectErr: true,
		},
		{
			name: "修改訂單狀態失敗",
			input: &order.ModifyOrderStatusReq{
				BusinessId: "B123",
				Status:     3,
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬狀態更新失敗
				mockOrderModel.EXPECT().UpdateStatus(gomock.Any(), gomock.Any()).Return(nil, errors.New("update failed")).Times(1)
			},
			expectErr: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 準備測試環境
			tt.prepare()

			// 初始化 ModifyOrderStatusLogic
			logic := NewModifyOrderStatusLogic(context.TODO(), svcCtx)

			// 執行 ModifyOrderStatus
			resp, err := logic.ModifyOrderStatus(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, resp)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.expectedRes, resp)
			}
		})
	}
}
